package eu.eudat.model.persist.tenantconfiguration;

import eu.eudat.commons.XmlHandlingService;
import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.convention.ConventionService;
import eu.eudat.data.*;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.deposit.DepositSourcePersist;
import gr.cite.tools.validation.ValidatorFactory;
import gr.cite.tools.validation.specification.Specification;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class DepositTenantConfigurationPersist {
	private List<DepositSourcePersist> sources;
	public static final String _sources = "sources";

	public List<DepositSourcePersist> getSources() {
		return sources;
	}

	public void setSources(List<DepositSourcePersist> sources) {
		this.sources = sources;
	}

	@Component(DepositTenantConfigurationPersist.DepositTenantConfigurationPersistValidator.ValidatorName)
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public static class DepositTenantConfigurationPersistValidator extends BaseValidator<DepositTenantConfigurationPersist> {

		public static final String ValidatorName = "DepositTenantConfigurationPersistValidator";

		private final MessageSource messageSource;

		private final ValidatorFactory validatorFactory;


		protected DepositTenantConfigurationPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
			super(conventionService, errors);
			this.messageSource = messageSource;
			this.validatorFactory = validatorFactory;
		}

		@Override
		protected Class<DepositTenantConfigurationPersist> modelClass() {
			return DepositTenantConfigurationPersist.class;
		}

		@Override
		protected List<Specification> specifications(DepositTenantConfigurationPersist item) {
			return Arrays.asList(
					this.spec()
							.must(() -> !this.isListNullOrEmpty(item.getSources()))
							.failOn(DepositTenantConfigurationPersist._sources).failWith(messageSource.getMessage("Validation_Required", new Object[]{DepositTenantConfigurationPersist._sources}, LocaleContextHolder.getLocale())),
					this.refSpec()
							.iff(() -> !this.isListNullOrEmpty(item.getSources()))
							.on(DepositTenantConfigurationPersist._sources)
							.over(item.getSources())
							.using(() -> this.validatorFactory.validator(DepositSourcePersist.DepositSourcePersistValidator.class))
			);
		}
	}
}
